import AWS from 'aws-sdk'
import awsJson from './aws.json'
import dimensions from './dimensions'

const dynamoClient = new AWS.DynamoDB.DocumentClient({
  credentials: awsJson["awsParams"].credentials,
  region: awsJson["awsParams"].region,
  apiVersion: awsJson["dynamoParams"].apiVersion
})

const s3 = new AWS.S3({
  credentials: awsJson["awsParams"].credentials,
  region: awsJson["awsParams"].region,
  apiVersion: awsJson["s3Params"].apiVersion
})

function getPages(query, keys, hits, finish, onErr) {
  dynamoClient.scan(query).eachPage(function (err, data, done) {
    if (err) {
      console.log(err, err.stack);
      onErr(err)
    } 
    else {
      hits += data.Count;
      if (data.LastEvaluatedKey) {
        keys.push(data.LastEvaluatedKey);
        query.ExclusiveStartKey = data.LastEvaluatedKey;
        getPages(query, keys, hits, finish, onErr);
      } 
      else {
        finish(keys, hits)
      }
    }
  })
}

function getPageData(query, pageKeys, pageIndex) {
  return new Promise((resolve, reject) => {
    query.ExclusiveStartKey = pageIndex != null ? pageKeys[pageIndex] : null
    console.log("pageData query:", query)
    var measurements = [];
    dynamoClient.scan(query).eachPage((err, data, done) => {
      console.log("data:", data)
      if (data != null) { //If any items returned
        data.Items.forEach(element => {
          let item = {}
          item.MeasurementID = element.MeasurementID;
          for (const d of dimensions.keys) {
            item[d] = element[d]
          }
          measurements.push(item);
        })
      } 
      else if (err) { //If error returned
        console.log(err, err.stack);
        reject(err)
      }
      resolve(measurements)
    })
  })
}
// queries dynamodb using optional search params
// onFinished is called with total records and a function that can be used
// to query a page of records
// onErr is called with err messsage on failure
function query(searchParams, itemsPerPage, onFinished, onErr) {
  console.log('querying with:', searchParams)

  let dataQuery = {
    TableName: awsJson["dynamoParams"].tableName,
    Limit: itemsPerPage
  }
  let pagesQuery = {
    TableName: awsJson["dynamoParams"].tableName,
    Limit: itemsPerPage,
    Select: "COUNT"
  }

  var filterExpression = "";
  var expressionAttributeValues = {};
  for (var key in searchParams) {
    if (searchParams.hasOwnProperty(key)) {
      /*keyConditions[key] = {
          ComparisonOperator:"BETWEEN",
          AttributeValueList:[ {"S": attributeRanges[key].min.toString()}, {"S": attributeRanges[key].max.toString()} ]
      }*/
      if (filterExpression) {
        filterExpression += " AND "
      }
      filterExpression += key + " BETWEEN :" + key + "Min and :" + key + "Max";
      expressionAttributeValues[":" + key + "Min"] = searchParams[key].min;
      expressionAttributeValues[":" + key + "Max"] = searchParams[key].max;
    }
  }

  if (filterExpression) {
    dataQuery.FilterExpression = filterExpression
    dataQuery.ExpressionAttributeValues = expressionAttributeValues
    pagesQuery.FilterExpression = filterExpression
    pagesQuery.ExpressionAttributeValues = expressionAttributeValues
  }

  var totalHits = 0
  var lastEvaluatedKeys = [null]
  getPages(pagesQuery, lastEvaluatedKeys, totalHits, (pageKeys, totalHits) => {
    console.log(totalHits, 'items,', pageKeys.length, 'pages')
    onFinished(
      totalHits, 
      pageKeys.length, 
      (pageIndex = null) => getPageData(dataQuery, pageKeys, pageIndex))
  }, onErr)
}

function getModelObjects(measurementID) {
  //Set required var
  var keyBase = measurementID + "/" + measurementID;

  //Set get params
  var modelGetParams = {
      Bucket: awsJson["s3Params"].bucketName,
      Key: keyBase + ".obj"
  }
  var isomapGetParams = {
      Bucket: awsJson["s3Params"].bucketName,
      Key: keyBase + ".isomap.png"
  }
  var materialGetParams = {
      Bucket: awsJson["s3Params"].bucketName,
      Key: keyBase + ".mtl"
  }
  //Make s3 promises
  var getModel = s3.getObject(modelGetParams).promise()
      .then(response => {
          return {
              type: "model",
              data: response.Body
          }
      })      
      .catch(err => {
        console.log("Server returned " + err)
      });
  var getIsomap = s3.getObject(isomapGetParams).promise()
      .then(response => {
          return {
              type: "isomap",
              data: response.Body
          }
      })
      .catch(err => {
        console.log("Server returned " + err)
      });
  var getMaterial = s3.getObject(materialGetParams).promise()
      .then(response => {
          return {
              type: "material",
              data: response.Body
          }
      })
      .catch(err => {
        console.log("Server returned " + err)
      });

  return Promise.all([getModel, getIsomap, getMaterial]).then(values => {
      var data = {};
      values.forEach(value => {
          switch(value["type"]) {
              case "model":
                  data["model"] = value.data.toString('utf8');
                  break;
              case "isomap":
                  data["isomap"] = "data:image/png;base64," + value.data.toString('base64');
                  break;
              case "material":
                  data["material"] = value.data.toString('utf8');
                  break;
              default:
                  console.log("Error parsing promise! [" + value["type"] + "]");
                  break;
          }
      });
      //Make s3 promises
      var modelUrl = s3.getSignedUrl('getObject', modelGetParams);
      var isomapUrl = s3.getSignedUrl('getObject', isomapGetParams);
      var materialUrl = s3.getSignedUrl('getObject', materialGetParams);

      data["modelUrl"] = modelUrl;
      data["isomapUrl"] = isomapUrl;
      data["materialUrl"] = materialUrl;
      
      return data;
  })
}

export default {
  query: query,
  getModelObjects: getModelObjects
}